/* This program takes a Process ID (PID) as comand line argument
 * and outputs all 100ms the memory which is occupied by the
 * program with the specific PID.
 */
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <ctime>
#include <cstdio>
#include <string>
#include <chrono>
#include <unistd.h>

#ifdef __APPLE__
// Mac OS solution inspired by
// http://miknight.blogspot.com.au/2005/11/resident-set-size-in-mac-os-x.html
#include <sys/sysctl.h>
#include <mach/task.h>
#include <mach/mach_init.h>
void getres(task_t task, unsigned int *rss)
{
	struct task_basic_info t_info;
	mach_msg_type_number_t t_info_count = TASK_BASIC_INFO_COUNT;

	task_info(task, TASK_BASIC_INFO, (task_info_t)&t_info, &t_info_count);
	*rss = t_info.resident_size;
}
#endif

using namespace std;
using namespace std::chrono;


int main(int argc, char **argv){
#ifdef __APPLE__
	uint64_t peak = 0;
	pid_t pid = argc > 1 ? atoi(argv[1]) : 1;
	uint32_t rss, psize;
	psize = getpagesize();
	task_t task = MACH_PORT_NULL;

	while(  task_for_pid(current_task(), pid, &task) == KERN_SUCCESS ){
		usleep(100000); // wait 100ms
		getres(task, &rss);
		if(peak < rss * psize)
		{
			peak = rss * psize;
		}
	}
#else
	int64_t peak = 0;
	if(argc != 2) {
		cerr << "Usage: " << argv[0] << " PID" << endl;
		return 1;
	}
	string pid(argv[1]);
	if ( system(NULL) ) {
		if(system("getconf PAGESIZE > pagesize.txt")) {
			cout << "Something seems to be wrong!" << endl;
			return 1;
		}
		int pagesize = 0;
		{
			ifstream pagesize_in("pagesize.txt");
			pagesize_in >> pagesize;
			pagesize_in.close();
			remove("pagesize.txt");
		}
		while ( 1 ) {
			usleep(100000); // wait 100ms
			ifstream statm_in( ("/proc/"+pid+"/statm").c_str() );
			if ( !statm_in )
				break;
			long long pmem[3];
			statm_in >> pmem[0] >> pmem[1] >> pmem[2];
			long long ures = pmem[1] - pmem[2];
			if(peak < ures * pagesize)
			{
				peak = ures * pagesize;
			}
		}
	} else {
		cout<<"No shell available!"<<endl;
		return 1;
	}
#endif
	cout << peak << endl;

	return 0;
}
